@@ -2,6 +2,8 @@
 
 namespace XploreBooking\Controller;
 
+use XploreBooking\Post\JourneyDate;
+
 class BookingController
 {
     const SEATS = 29;
@@ -39,10 +41,20 @@ class BookingController
                 'callback' => [$this, 'cancelUnconfirmedBookings'],
             ]);
 
+            register_rest_route('xplore-booking/v1', '/cancel-pending-bookings/', [
+                'methods' => 'GET',
+                'callback' => [$this, 'cancelPendingBookings'],
+            ]);
+
             register_rest_route('xplore-booking/v1', '/validate-recaptcha/', [
                 'methods' => 'POST',
                 'callback' => [$this, 'validateRecaptcha'],
             ]);
+
+            register_rest_route('xplore-booking/v1', '/check-booking/', [
+                'methods' => 'POST',
+                'callback' => [$this, 'checkBooking'],
+            ]);
         });
 
         add_action('template_redirect', [$this, 'confirmationRedirect']);
@@ -58,13 +70,176 @@ class BookingController
         add_action('pre_get_posts', [$this, 'postFilter']);
 
         add_filter('rest_pre_dispatch', [$this, 'rest_basic_auth_check_errors'], 10, 3);
+
+        add_filter('wp_insert_post_data', [$this, 'preSaveBookings'], 99, 2);
+
+        add_action('wp_trash_post', [$this, 'moveBookingToTrash'], 10, 1);
+
+        add_action('untrash_post', [$this, 'moveBookingToActive'], 10, 2);
+    }
+
+    public function moveBookingToTrash($postId){
+        /*
+        * function to do the following
+        * If booking is moved to trash then:
+        * get total_seats of booking and remove to journey date
+        */
+
+        $postData = get_post($postId);
+        if($postData->post_type == 'booking'){
+            $outboundJourney = get_post_meta($postId, 'outbound_journey', true);
+            $outboundJourney = (is_array($outboundJourney)) ? $outboundJourney[0] : null;
+            $inboundJourney = get_post_meta($postId, 'inbound_journey', true);
+            $inboundJourney = (is_array($inboundJourney)) ? $inboundJourney[0] : null;
+            $seats = get_post_meta($postId, 'total_seats', true);
+
+            if($outboundJourney){
+                $outboundJourneydate = get_post($outboundJourney);
+                $outboundCurrentSeats = get_post_meta($outboundJourneydate->ID, 'seats', true);
+                update_field('seats', $outboundCurrentSeats + $seats, $outboundJourneydate->ID);
+            }
+            if($inboundJourney){
+                $inboundJourneydate = get_post($inboundJourney);
+                $inboundCurrentSeats = get_post_meta($inboundJourneydate->ID, 'seats', true);
+                update_field('seats', $inboundCurrentSeats + $seats, $inboundJourneydate->ID);
+            }
+        }
+    }
+
+    public function moveBookingToActive($postId, $prevStatus){
+        /*
+        * function to do the following
+        * If booking is moved to trash then:
+        * get total_seats of booking and add to journey date
+        */
+        $postData = get_post($postId);
+        if($postData->post_type == 'booking'){
+            $outboundJourney = get_post_meta($postId, 'outbound_journey', true);
+            $outboundJourney = (is_array($outboundJourney)) ? $outboundJourney[0] : null;
+            $inboundJourney = get_post_meta($postId, 'inbound_journey', true);
+            $inboundJourney = (is_array($inboundJourney)) ? $inboundJourney[0] : null;
+            $seats = get_post_meta($postId, 'total_seats', true);
+
+            if($outboundJourney){
+                $outboundJourneydate = get_post($outboundJourney);
+                $outboundCurrentSeats = get_post_meta($outboundJourneydate->ID, 'seats', true);
+                update_field('seats', $outboundCurrentSeats - $seats, $outboundJourneydate->ID);
+            }
+            if($inboundJourney){
+                $inboundJourneydate = get_post($inboundJourney);
+                $inboundCurrentSeats = get_post_meta($inboundJourneydate->ID, 'seats', true);
+                update_field('seats', $inboundCurrentSeats - $seats, $inboundJourneydate->ID);
+            }
+        }
+    }
+
+    public function preSaveBookings($newData, $postarr){
+
+        /*
+        * function to do the following
+        * detect changes to journeys 
+        * if journey is changed - remove seating from old journey and append to new
+        * detect changes to seating
+        * if seating is changed - adjust seating on journey
+        */
+        if($postarr['post_type'] == 'booking'){
+            $journeyChanged = false;
+
+            $oldData = get_post($postarr['ID']);
+
+            $oldOutboundJourney = get_post_meta($postarr['ID'], 'outbound_journey', true);
+            $oldOutboundJourney = (is_array($oldOutboundJourney)) ? $oldOutboundJourney[0] : null;
+            $newOutboundJourney = $_POST['acf']['field_5cf5622486fd7'];
+            $newOutboundJourney = (is_array($newOutboundJourney)) ? $newOutboundJourney[0] : null;
+            $oldInboundJourney = get_post_meta($postarr['ID'], 'inbound_journey', true);
+            $oldInboundJourney = (is_array($oldInboundJourney)) ? $oldInboundJourney[0] : null;
+            $newInboundJourney = $_POST['acf']['field_5cf7f1b976a3b'];
+            $newInboundJourney = (is_array($newInboundJourney)) ? $newInboundJourney[0] : null;
+
+            $oldSeats = get_post_meta($postarr['ID'], 'total_seats', true);
+            $newSeats = $_POST['acf']['field_5e18912b1bd48'];
+            $amendmentSeats = intval($newSeats) - intval($oldSeats);
+
+            
+
+            if($oldOutboundJourney !== $newOutboundJourney){
+                $journeyChanged = true;
+                // outbound journey has changed
+                $oldOutboundjourneyDate = get_post($oldOutboundJourney);
+                $newOutboundJourneyDate = get_post($newOutboundJourney);
+                $oldOutboundCurrentSeats = get_post_meta($oldOutboundjourneyDate->ID, 'seats', true);
+                $newOutboundCurrentSeats = get_post_meta($newOutboundJourneyDate->ID, 'seats', true);
+                // adding seats back to old journey
+                update_field('seats', $oldOutboundCurrentSeats + $oldSeats, $newOutboundJourneyDate->ID);
+                // subtracting seats from new journey
+                update_field('seats', $newOutboundCurrentSeats - $newSeats, $newOutboundJourneyDate->ID);
+            }
+            if($oldInboundJourney !== $newInboundJourney){
+                $journeyChanged = true;
+                // inbound journey has changed
+                $oldInboundjourneyDate = get_post($oldInboundJourney);
+                $newInboundJourneyDate = get_post($newInboundJourney);
+                $oldInboundCurrentSeats = get_post_meta($oldInboundjourneyDate->ID, 'seats', true);
+                $newInboundCurrentSeats = get_post_meta($newInboundJourneyDate->ID, 'seats', true);
+                // adding seats back to old journey
+                update_field('seats', $oldInboundCurrentSeats + $oldSeats, $newInboundJourneyDate->ID);
+                // subtracting seats from new journey
+                update_field('seats', $newInboundCurrentSeats - $newSeats, $newInboundJourneyDate->ID);
+            }
+
+            if($amendmentSeats !== 0 && !$journeyChanged){
+                // seating allocation has changed
+                // if positive, seats to deduce from existing number. if negitive, seats to add from existing number
+                if($amendmentSeats > 0){
+                    // removing seats to inbound / outbound
+                    if($oldOutboundJourney){
+                        $outboundjourneyDate = get_post($oldOutboundJourney);
+                        $outboundCurrentSeats = get_post_meta($outboundjourneyDate->ID, 'seats', true);
+                        update_field('seats', $outboundCurrentSeats - $amendmentSeats, $outboundjourneyDate->ID);
+                    }
+                    if($oldInboundJourney){
+                        $inboundjourneyDate = get_post($oldInboundJourney);
+                        $inboundCurrentSeats = get_post_meta($inboundjourneyDate->ID, 'seats', true);
+                        update_field('seats', $inboundCurrentSeats - $amendmentSeats, $inboundjourneyDate->ID);
+                    }
+                }
+                if($amendmentSeats < 0){
+                    // adding seats to inbound / outbound
+                    if($oldOutboundJourney){
+                        $outboundjourneyDate = get_post($oldOutboundJourney);
+                        $outboundCurrentSeats = get_post_meta($outboundjourneyDate->ID, 'seats', true);
+                        update_field('seats', $outboundCurrentSeats + abs($amendmentSeats), $outboundjourneyDate->ID);
+                    }
+                    if($oldInboundJourney){
+                        $inboundjourneyDate = get_post($oldInboundJourney);
+                        $inboundCurrentSeats = get_post_meta($inboundjourneyDate->ID, 'seats', true);
+                        update_field('seats', $inboundCurrentSeats + abs($amendmentSeats), $inboundjourneyDate->ID);
+                    }
+                }
+            }
+
+            
+        }
+        return $newData;
+    }
+
+    public static function checkBooking($request){
+        $bookingID = $request['bookingId'];
+        $postDateAndTime = strtotime(get_the_date('', $bookingID). " ". get_the_time('', $bookingID));
+        $now = strtotime('-1 hours');
+        $bookingPost = get_post($bookingID);
+        if(!$bookingPost || $postDateAndTime < $now){
+            return new \WP_REST_Response('failed', 200);
+        } else {
+            return new \WP_REST_Response('success', 200);
+        }
     }
 
     public static function cancelUnconfirmedBookings()
     {
         $args = [
             'post_type' => 'booking',
-            'posts_per_page' => -1,
+            'numberposts' => -1,
             'meta_query' => [
                 [
                     'key' => 'payment_reference',
@@ -75,35 +250,95 @@ class BookingController
             ],
         ];
 
-        $pendingBookings = new \WP_Query($args);
-        if ($pendingBookings->have_posts()) {
-            while ($pendingBookings->have_posts()) {
-                $pendingBookings->the_post();
+        $pendingBookings = get_posts($args);
+        if ($pendingBookings) {
+            foreach ($pendingBookings as $post) {                
                 $now = strtotime('-2 days');
-                $date = strtotime(get_the_date());
+                $date = strtotime(get_the_date('', $post->ID));
                 if ($date < $now) {
-                    $tickets = get_field('tickets');
+                    $tickets = get_field('tickets', $post->ID);
                     $totalSeats = 0;
                     foreach ($tickets as $ticket) {
                         $realTicket = get_page_by_title($ticket['ticket_type'], null, 'ticket');
                         $seatsPerTicket = get_field('seats', $realTicket->ID) * $ticket['quantity'];
                         $totalSeats += $seatsPerTicket;
                     }
-                    $outboundJourney = get_field('outbound_journey');
-                    $inboundJourney = get_field('inbound_journey');
+                    $outboundJourneys = get_field('outbound_journey', $post->ID);
+                    $inboundJourneys = get_field('inbound_journey', $post->ID);
+                    
+                    foreach($outboundJourneys as $outboundJourney){
+                        $outboundSeats = get_field('seats', $outboundJourney) + $totalSeats;
+                        update_field('seats', $outboundSeats, $outboundJourney);
+                    }
 
-                    $outboundSeats = get_field('seats', $outboundJourney) + $totalSeats;
-                    $inboundSeats = get_field('seats', $inboundJourney) + $totalSeats;
+                    if($inboundJourneys){
+                        foreach($inboundJourneys as $inboundJourney){
+                            $inboundSeats = get_field('seats', $inboundJourney) + $totalSeats;
+                            update_field('seats', $inboundSeats, $inboundJourney);
+                        }
+                    }
 
-                    update_field('seats', $outboundSeats, $outboundJourney);
-                    update_field('seats', $inboundSeats, $inboundJourney);
-
-                    wp_delete_post(get_the_ID());
+                    wp_delete_post($post->ID);
                 }
             }
         }
 
-        wp_reset_query();
+        wp_reset_postdata();
+    }
+
+    public static function cancelPendingBookings()
+    {
+        $args = [
+            'post_type' => 'booking',
+            'numberposts' => -1,
+            'meta_query' => [
+                [
+                    'key' => 'booking_status',
+                    'value' => 'Pending',
+                    'compare' => '=',
+                ]
+            ],
+        ];
+
+        $pendingBookings = get_posts($args);
+        if ($pendingBookings) {
+            foreach ($pendingBookings as $post) {
+                $payment = get_field('payment_reference', $post->ID);
+                if($payment){
+                    continue;
+                }
+
+                $now = strtotime('-1 hours');
+                $date = strtotime(get_the_date('', $post->ID)." ".get_the_time('', $post->ID));
+                if ($date < $now) {
+                    $tickets = get_field('tickets', $post->ID);
+                    $totalSeats = 0;
+                    foreach ($tickets as $ticket) {
+                        $realTicket = get_page_by_title($ticket['ticket_type'], null, 'ticket');
+                        $seatsPerTicket = get_field('seats', $realTicket->ID) * $ticket['quantity'];
+                        $totalSeats += $seatsPerTicket;
+                    }
+                    $outboundJourneys = get_field('outbound_journey', $post->ID);
+                    $inboundJourneys = get_field('inbound_journey', $post->ID);                    
+                    
+                    foreach($outboundJourneys as $outboundJourney){
+                        $outboundSeats = get_field('seats', (int)$outboundJourney) + $totalSeats;
+                        update_field('seats', $outboundSeats, (int)$outboundJourney);
+                    }
+
+                    if($inboundJourneys){
+                        foreach($inboundJourneys as $inboundJourney){
+                            $inboundSeats = get_field('seats', (int)$inboundJourney) + $totalSeats;
+                            update_field('seats', $inboundSeats, (int)$inboundJourney);
+                        }
+                    }
+
+                    wp_delete_post($post->ID);
+                }
+            }
+        }
+
+        wp_reset_postdata();
     }
 
     public function rest_basic_auth_check_errors($result, $server, $request)
@@ -215,6 +450,21 @@ class BookingController
             case 'failed':
                 update_field('payment_reference', $paymentReference, $bookingId);
                 update_field('booking_status', 'Failed', $bookingId);
+
+                // Restore Seats on the journey
+                $seats = get_field('total_seats', $bookingId);
+
+                $outboundJourney = get_field('outbound_journey', $bookingId);
+                $journeySeats = get_field('seats', $outboundJourney[0]);
+
+                update_field('seats', $journeySeats + $seats, $outboundJourney[0]);                 
+
+                $inboundJourney = get_field('inbound_journey', $bookingId);
+                if ($inboundJourney) {
+                    $journeySeats = get_field('seats', $inboundJourney[0]);
+                    update_field('seats', $journeySeats + $seats, $inboundJourney[0]);                    
+                }
+                // End Restore Seats on the journey
                 self::emailUserBookingDetails($status, $bookingId);
                 break;
             default:
@@ -432,7 +682,12 @@ class BookingController
         if ($bookingId == null) {
             return new \WP_REST_Response('No id was sent', 400);
         }
+        $bookingStatus = get_field('booking_status', $bookingId);
 
+        if($bookingStatus == "Failed"){
+            return ["status" => "failed"];
+        }
+        
         $outboundJourneyId = get_field('outbound_journey', $bookingId);
         $outboundRef = get_field('outbound_reference', $bookingId);
         $response[] = self::getFormattedJourneyText(get_the_title($outboundJourneyId[0]), $outboundRef);
@@ -557,6 +812,7 @@ class BookingController
             update_field('price', $price, $newBooking);
             update_field('total_seats', $seats, $newBooking);
             update_field('outbound_booking_time', $outboundDate, $newBooking);
+            update_field('booking_status', 'Pending', $newBooking);
 
             if ($includeInbound) {
                 update_field('inbound_booking_time', $inboundDate, $newBooking);
